/*****************************************************************************
 *   adctest.c:  main C entry file for NXP LPC29xx Family Microprocessors
 *
 *   Copyright(C) 2007, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2007.08.16  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/

#include <string.h>
#include <limits.h>
#include <stdio.h>
#include "LPC29xx.h"                        /* LPC29xx definitions */
#include "type.h"
#include "adc.h"
#include "target.h"
#include "irq.h"
#include <intrinsics.h> /* for interrupts iar */
#include "timer.h"
#include "drv_hd44780.h"

extern volatile DWORD timer_counter;

extern volatile DWORD ADC2Value[ITERATION][ADC_NUM];
extern volatile DWORD ADC2ValueAverage[ADC_NUM];
extern volatile DWORD ADC2IntDone;
extern volatile DWORD ADC2Iteration;

static void setled(int led, int state)
{
	switch(led) {
		case 3:
			if(state) {
				GPIO0_OR &= ~(1<<28);
			} else {
				GPIO0_OR |= (1<<28);
			}
			break;
		case 1:
			if(state) {
				GPIO3_OR &= ~(1<<0);
			} else {
				GPIO3_OR |= (1<<0);
			}
			break;
		case 2:
			if(state) {
				GPIO3_OR &= ~(1<<12);
			} else {
				GPIO3_OR |= (1<<12);
			}
			break;
	}
}

void Dly100us(void *arg)
{
	for(volatile int i=0; i<10*1000; ++i);
}

static void animation_delay(void)
{
	timer_counter = 0;
	while(timer_counter < 1);
}

static void animate_powerup(void)
{
	HD44780_StrShow(1,1,"________");
	animation_delay();
	HD44780_StrShow(1,1,"S_______");
	animation_delay();
	HD44780_StrShow(1,1,"_T______");
	animation_delay();
	HD44780_StrShow(1,1,"__A_____");
	animation_delay();
	HD44780_StrShow(1,1,"___R____");
	animation_delay();
	HD44780_StrShow(1,1,"____T___");
	animation_delay();
	HD44780_StrShow(1,1,"_____I__");
	animation_delay();
	HD44780_StrShow(1,1,"______N_");
	animation_delay();
	HD44780_StrShow(1,1,"_______G");
	animation_delay();
}


/*****************************************************************************
**   Main Function  main()
******************************************************************************/
int main (void)
{
  HD44780_ERROR_CODE_DEF lcdinit_st = HD44780_OK;
  int adc_val=0;

  /* Enable MSCSS timer for ADC event trigger, match0 for ADC0 */
  init_timer(TIMER_TICKS);
  enable_timer();

  ADC2_Init();
  ADC2_Start();

  /* GPIO 3.0 and 3.13 are LED pins set to output */
  GPIO0_DR = (1<<28);
  GPIO3_DR = (1<<0)|(1<<12);
  SFSP0_28 = (1<<2);
  SFSP3_0 = (1<<2);
  SFSP3_13 = (1<<2);

  SFSP2_26 = (1<<2);
  SFSP2_27 = (1<<2);

  setled(1, 0);
  setled(2, 0);
  setled(3, 1);

 __enable_interrupt(); // iar

  /* HD44780 LCD driver init */
  lcdinit_st = HD44780_PowerUpInit();
  if(lcdinit_st != HD44780_OK) {
	  for(;;) {
		  setled(1,1);
		  setled(2,1);
		  setled(3,1);
	  }
  }

  animate_powerup();
  for(;;) {
	 if ( ADC2IntDone )
	 {
		 adc_val = ADC2Value[0][0];
		 ADC2IntDone = 0;
		 ADC2_Start();
		
     char  buf[8]	;
     
     sprintf(buf,"%3d%%",(adc_val*100)/(1024-1));
     /*Clear LCD*/
     HD44780_StrShow(1,1,(const HD44780_STRING_DEF *) "        ");
     /*Get string length*/
     int strlength = strlen(buf);
     /*Display temperature on LCD*/
     HD44780_StrShow(1+8-strlength,1,(const HD44780_STRING_DEF *) buf);
  }
 }
  //return 0;
}

/*****************************************************************************
**                            End Of File
*****************************************************************************/

